package wsn.counter.client;

import java.io.FileInputStream;
import java.util.List;

import javax.xml.namespace.QName;
import javax.xml.rpc.Stub;

import org.apache.axis.message.addressing.EndpointReferenceType;

import org.oasis.wsn.NotificationProducer;
import org.oasis.wsn.Subscribe;
import org.oasis.wsn.SubscribeResponse;
import org.oasis.wsn.TopicExpressionType;
import org.oasis.wsrf.lifetime.Destroy;

import org.globus.wsrf.encoding.ObjectSerializer;
import org.globus.wsrf.core.notification.SubscriptionManager;
import org.globus.wsrf.core.notification.service.SubscriptionManagerServiceAddressingLocator;

import org.globus.axis.util.Util;
import org.globus.wsrf.NotifyCallback;
import org.globus.wsrf.NotificationConsumerManager;
import org.globus.wsrf.WSNConstants;
import org.globus.wsrf.encoding.ObjectDeserializer;

import org.xml.sax.InputSource;

import wsn.counter.Counter;
import wsn.counter.impl.CounterResource;
import wsn.counter.service.CounterServiceAddressingLocator;

public class CounterSubscribe implements NotifyCallback {
    static {
        Util.registerTransport();
    }

    private NotificationConsumerManager consumer = null;
    private EndpointReferenceType consumerEPR = null;
    private EndpointReferenceType subscriptionEPR = null;
    
    public CounterSubscribe(EndpointReferenceType epr, String topic) throws Exception {        
        // Start local notification consumer
        this.consumer = NotificationConsumerManager.getInstance();
        this.consumer.startListening();
        this.consumerEPR = consumer.createNotificationConsumer(this);

        // Prepare subscribe request: consumer epr, topic name, etc.
        Subscribe request = new Subscribe();
        request.setUseNotify(Boolean.TRUE);
        request.setConsumerReference(this.consumerEPR);
        TopicExpressionType topicExpression = new TopicExpressionType();
        topicExpression.setDialect(WSNConstants.SIMPLE_TOPIC_DIALECT);
        topicExpression.setValue(QName.valueOf(topic));
        request.setTopicExpression(topicExpression);

        CounterServiceAddressingLocator locator =
            new CounterServiceAddressingLocator();
        Counter counter = locator.getCounterPort(epr);        
        SecurityHelper.setSecurity((Stub) counter);

        // Perform subscription against counter WS-Resource.
        // An EPR is returned to the subscription WS-Resource that represents
        // this subscription. Subscription lifetime is managed through this
        // WS-Resource.
        SubscribeResponse subscribeResponse = counter.subscribe(request);
        this.subscriptionEPR = subscribeResponse.getSubscriptionReference();
        
        System.out.println("Subscription done");
    }

    public void deliver(List topicPath,
                        EndpointReferenceType producer,
                        Object message) {
        try {
            Object topic = topicPath.get(0);
            System.out.println("Received notification on topic: " + topic.toString() + ":");
            System.out.println("Value:\n" + ObjectSerializer.toString(
                                   message, QName.valueOf(topic.toString())
                               ));
        } catch (Exception e) {
            System.err.println("Failed to display notification: " + e);
        }
    }

    public void stop() throws Exception {
        // Destroy server-side subscription WS-Resource
        SubscriptionManagerServiceAddressingLocator locator =
            new SubscriptionManagerServiceAddressingLocator();
        SubscriptionManager manager = locator.getSubscriptionManagerPort(
            this.subscriptionEPR
        );
        SecurityHelper.setSecurity((Stub) manager);        
        manager.destroy(new Destroy());
        
        this.consumer.removeNotificationConsumer(this.consumerEPR);
        this.consumer.stopListening();
    }
    
    public static void main(String[] args) throws Exception {       
	try {
            String eprFile = args[0];
            String topic = args[1];

            // Get endpoint reference of counter WS-Resource
            FileInputStream fis = new FileInputStream(eprFile);
            EndpointReferenceType endpoint = (EndpointReferenceType)
                ObjectDeserializer.deserialize(
                    new InputSource(fis), EndpointReferenceType.class
                );
            fis.close();

            CounterSubscribe subscriber = new CounterSubscribe(endpoint, topic);
            System.out.println("Waiting for notifications ... ");
            System.in.read();
            subscriber.stop();
	} catch (ArrayIndexOutOfBoundsException e) {
            System.err.println("Usage: CounterSubscribe <EPR file> <topic-qname>");
	}
	    
    }
}
