package wsn.counter.impl;

import java.rmi.RemoteException;

import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.wsrf.ResourceContext;
import org.globus.wsrf.ResourceHome;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.utils.AddressingUtils;

import org.oasis.wsrf.faults.BaseFaultType;

import wsn.counter.CreateCounterRequest;
import wsn.counter.CreateCounterResponse;
import wsn.counter.IncrementRequest;
import wsn.counter.IncrementResponse;
import wsn.counter.NegativeIncrementFaultType;

public class CounterService {    
    private static Log logger = LogFactory.getLog(CounterResource.class.getName());

    /**
     * A WS-Resource factory operation. Creates a new counter WS-Resource.
     * Returns an Endpoint Reference (EPR), which is basically a
     * network-pointer to the new WS-Resource.     
     */
    public CreateCounterResponse createCounter(CreateCounterRequest parameters)
        throws RemoteException, BaseFaultType {
        ResourceContext ctx = ResourceContext.getResourceContext();
        CounterHome home = (CounterHome) ctx.getResourceHome();

        EndpointReferenceType epr = null;
        try {        
            ResourceKey key = home.create();
            epr = AddressingUtils.createEndpointReference(ctx, key);
        } catch (Exception e) {
            throw new RemoteException("Failed to create epr: ", e);
        }
        
        return new CreateCounterResponse(epr);
    }

    /**
     * An operation that increments a counter WS-Resource
     * (identified by the EPR that was used in the invocation).
     */    
    public IncrementResponse increment(IncrementRequest incr)
        throws RemoteException, BaseFaultType, NegativeIncrementFaultType {
        CounterResource resource = (CounterResource)
            ResourceContext.getResourceContext().getResource();        
        int[] values = incr.getValue();
        for(int i = 0; i < values.length; i++) {
            if (values[i] < 0) {
                throw new NegativeIncrementFaultType();
            }            
            resource.increment(values[i]);
        }
        
        return new IncrementResponse();
    }   
}
